from flask import Flask, render_template, request, jsonify
import swisseph as swe
from datetime import datetime
from geopy.geocoders import Nominatim
from geopy.exc import GeocoderTimedOut

app = Flask(__name__)

# Initialize geolocator
geolocator = Nominatim(user_agent="astrochart_app")


@app.route('/')
def index():
    return render_template('index.html')


@app.route('/generate_chart', methods=['POST'])
def generate_chart():
    try:
        # Retrieve the data from the form
        data = request.json
        name = data['name']
        dob = data['dob']
        birth_time = data['birth_time']
        place = data['place']

        # Parse the Date and Time
        birth_datetime_str = f"{dob} {birth_time}"
        birth_datetime = datetime.strptime(birth_datetime_str, "%Y-%m-%d %H:%M")
        jd_ut = swe.julday(birth_datetime.year, birth_datetime.month, birth_datetime.day,
                           birth_datetime.hour + birth_datetime.minute / 60.0)

        # Fetch latitude and longitude from place
        location = geolocator.geocode(place)
        if location:
            latitude = location.latitude
            longitude = location.longitude
        else:
            return jsonify({"error": "Location not found!"}), 400

        # Generate planetary positions and cusps
        planet_positions = calculate_planet_positions(jd_ut)
        cusps = calculate_cusps(jd_ut, latitude, longitude)  # Use the fetched lat/lon

        # Prepare the data in the required format
        data_radix = {
            "planets": planet_positions,
            "cusps": cusps
        }

        return jsonify(data_radix)

    except Exception as e:
        return jsonify({"error": str(e)}), 400


@app.route('/cities', methods=['GET'])
def get_cities():
    query = request.args.get('q', '')
    if not query:
        return jsonify([])

    try:
        locations = geolocator.geocode(query, exactly_one=False, limit=10)
        if locations:
            cities = [location.address for location in locations]
            return jsonify(cities)
        else:
            return jsonify([])
    except GeocoderTimedOut:
        return jsonify({"error": "Geocoder service timed out. Please try again."}), 500
    except Exception as e:
        return jsonify({"error": str(e)}), 500




def calculate_planet_positions(jd_ut):
    planets = {
        'Sun': swe.SUN,
        'Moon': swe.MOON,
        'Mercury': swe.MERCURY,
        'Venus': swe.VENUS,
        'Mars': swe.MARS,
        'Jupiter': swe.JUPITER,
        'Saturn': swe.SATURN,
        'Uranus': swe.URANUS,
        'Neptune': swe.NEPTUNE,
        'Pluto': swe.PLUTO
    }

    positions = {}
    for planet_name, planet_id in planets.items():
        position, _ = swe.calc(jd_ut, planet_id)
        positions[planet_name] = [position[0]]  # Longitude of the planet

    return positions


def calculate_cusps(jd_ut, latitude, longitude):
    # Fetch house cusps and additional data based on the provided lat/lon
    cusps, ascmc = swe.houses(jd_ut, latitude, longitude, b'P')  # 'P' for Placidus system
    return cusps[:12]  # Return the first 12 cusps


if __name__ == '__main__':
    app.run(debug=True)